local Grid2StatusAuraGroup = Grid2:NewModule("StatusAuraGroup")

--{{{  Default Setup

-- {{displaySpellId, <false = any>|<true = mine>|<blinkThreshold>, r, g, b[, r, g, b]...}, spellId1, spellId2, ...}
function Grid2:SetupDefaultAuraGroups(setup, class)
	local setupIndicator = setup.status

	if (Grid2.healers[class]) then
		setup.auraGroups["group-hots"] = {{139, false, nil, .9, .3, .8, 1}, 139, 740, 774, 8936, 33763, 48438}

		self:SetupIndicatorStatus(setupIndicator, "side-left", "group-hots", 89)
	end

	if (class == "DRUID") then
		setup.auraGroups["group-GiftOfTheWild"] = {{26991, false, true, 1, 1, 0, 1}, 26991, 26990}
		setup.auraGroups["group-Swiftmend"] = {{18562, false, nil, .2, .6, 1, 1}, 774, 8936}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-GiftOfTheWild", 69)
		self:SetupIndicatorStatus(setupIndicator, "side-left", "group-Swiftmend", 99)
	elseif (class == "MAGE") then
		setup.auraGroups["group-DalaranBrilliance"] = {{61316, false, true, 0, 0, 1, 1}, 61316, 27126, 27127, 61024}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-DalaranBrilliance", 99)
	elseif (class == "PALADIN") then
		setup.auraGroups["group-BlessingOfKings"] = {{25898, false, true, 1, 0, .8, 1}, 20217, 25898}
		setup.auraGroups["group-BlessingOfMight"] = {{27141, false, true, 1, 0, .8, 1}, 27140, 27141}
		setup.auraGroups["group-BlessingOfSanctuary"] = {{25899, false, true, 1, 0, .8, 1}, 20911, 25899}
		setup.auraGroups["group-BlessingOfWisdom"] = {{27143, false, true, 1, 0, .8, 1}, 27142, 27143}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfKings", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfMight", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfSanctuary", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-BlessingOfWisdom", 99)
	elseif (class == "PRIEST") then
		setup.auraGroups["group-ShadowProtection"] = {{39374, false, true, 1, 0, .8, 1}, 25433, 39374}
		setup.auraGroups["group-DivineSpirit"] = {{32999, false, true, 1, 0, .8, 1}, 27841, 32999}
		setup.auraGroups["group-PrayerOfFortitude"] = {{25392, false, true, 1, 0, .8, 1}, 25389, 25392}

		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-ShadowProtection", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-DivineSpirit", 99)
		self:SetupIndicatorStatus(setupIndicator, "side-bottom", "group-PrayerOfFortitude", 99)
	end
end

-- {{displaySpellId, <false = any>|<true = mine>|<blinkThreshold>, r, g, b[, r, g, b]...}, spellId1, spellId2, ...}
function Grid2:SetupDefaultAuraGroupDebuffs(setup, class)
	local setupIndicator = setup.status
	setup.auraGroupDebuffs["healing-prevented"] = {{55593, false, .9, 0, .8, 0.75}, 41292, 30843, 55593, 45996}
	setup.auraGroupDebuffs["healing-reduced"] = {{12294, false, .9, 0, .4, 1}, 19434, 56112, 12294, 13218, 13222, 13223, 13224, 27189, 57974, 57975, 32858, 40599, 23169, 7068, 13583, 34073, 32378, 30423, 44534, 32315, 19716, 54378, 28440, 28776, 54121, 36023, 45885, 45347, 36917, 34366, 25646, 35189, 59525, 22859}

	self:SetupIndicatorStatus(setupIndicator, "alpha", "healing-prevented", 89)
	self:SetupIndicatorStatus(setupIndicator, "border", "healing-reduced", 89)
	self:SetupIndicatorStatus(setupIndicator, "corner-bottom-right", "healing-reduced", 89)
end
--/dump Grid2.statuses["healing-reduced"]:IsActive("player")

local prev_MakeDefaultSetup = Grid2.MakeDefaultSetup
function Grid2:MakeDefaultSetup(...)
	local setup, class = prev_MakeDefaultSetup(self, ...)

	if (Grid2.healers[class]) then
		self:SetupIndicatorTypeLocation(setup, "square", "side-left", "side-left", {7, "LEFT", "LEFT", 1, 0})
	end
	if (not setup.auraGroups) then
		setup.auraGroups = {}
		self:SetupDefaultAuraGroups(setup, class)
	end
	if (not setup.auraGroupDebuffs) then
		setup.auraGroupDebuffs = {}
		self:SetupDefaultAuraGroupDebuffs(setup, class)
	end

	return setup, class
end

--}}}


--{{{  Status Creation

-- UpdateState for all Auras in the Group
local function status_UpdateState(self, unit, auraName, iconTexture, count, duration, expiration)
	if (self.auraNames[auraName]) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
		self.new_expiration = expiration
	end
end

-- UpdateState for only my Auras in the Group
local function status_UpdateStateMine(self, unit, auraName, iconTexture, count, duration, expiration, isMine)
	if (self.auraNames[auraName] and isMine) then
		self.states[unit] = true
		self.textures[unit] = self.iconTexture
		self.counts[unit] = count
		self.durations[unit] = duration
		self.expirations[unit] = expiration
		self.new_state = self:IsActive(unit)
		self.new_count = count
		self.new_expiration = expiration
	end
end

-- Buff type group
function Grid2:SetupStatusAuraGroupBuff(infoAuraGroup)
	local info = infoAuraGroup[1]
	local status = self:CreateBuffStatus(unpack(info))

	local mine = info[2]
	status.UpdateState = mine and status_UpdateStateMine or status_UpdateState

	return status
end

-- Debuff type group
function Grid2:SetupStatusAuraGroupDebuff(infoAuraGroup)
	local info = infoAuraGroup[1]
	local status = self:CreateDebuffStatus(unpack(info))

	status.UpdateState = status_UpdateState

	return status
end

-- AuraGroup is implemented as a regular Aura status with modified detection code for UpdateState
function Grid2:SetupStatusAuraGroup(status, statusKey, infoAuraGroup)
	local info = infoAuraGroup[1]
	local displaySpellId = info[1]
	status.name = statusKey -- force name

	status.defaultDB.profile.displaySpellId = displaySpellId
	_, _, status.iconTexture = GetSpellInfo(displaySpellId)

	local auraNames = {}
	status.auraNames = auraNames
	for index = 2, # infoAuraGroup do
		local spellName = infoAuraGroup[index]
		if (type(spellName) == "number") then
			spellName = GetSpellInfo(spellName)
		end
		assert(type(spellName) == "string")
		auraNames[spellName] = true
	end

	self:RegisterStatus(status, { "color", "icon", "percent" })
	self:UpdateColorHandler(status)
	self:UpdateBlinkHandler(status)
	return status
end

local logItems = {}	-- n = startTime

function Grid2:LogEvent(eventName, arg1)
	if (AutoBar.db.account.performance or AutoBar.db.account.logEvents) then
		if (arg1) then
			print(eventName, "arg1" , arg1, "time:", GetTime())
		else
			print(eventName, "time:", GetTime())
		end
	end
end

function Grid2:LogEventStart(eventName)
	if (true) then
		if (logItems[eventName]) then
			print(eventName, "restarted before previous completion")
		else
			logItems[eventName] = GetTime()
			print(eventName, "started time:", logItems[eventName])
		end
	end
end

function Grid2:LogEventEnd(eventName, ...)
	if (true) then
		if (logItems[eventName]) then
			local elapsed = GetTime() - logItems[eventName]
			logItems[eventName] = nil
			print(eventName, ..., "time:", elapsed)
		else
			print(eventName, "restarted before previous completion")
		end
	end
end

local function test()
	local table0 = {26990, 774, 8936, 61316, 27126, 27127, 61024, 20217, 25898, 27140}
	local table1 = {19434, 56112, 12294, 13218, 13222, 13223, 13224, 27189, 57974, 57975}
	local table2 = {19434, 56112, 12294, 13218, 13222, 13223, 13224, 27189, 57974, 57975, 32858, 40599, 23169, 7068, 13583, 34073, 32378, 30423, 44534, 32315, 19716, 54378, 28440, 28776, 54121, 36023, 45885, 45347, 36917, 34366, 25646, 35189, 59525, 22859}

	local t1 = {}
	local t2 = {}
	for i, spellId in ipairs(table1) do
		t1[spellId] = true
	end
	for i, spellId in ipairs(table2) do
		t2[spellId] = true
	end

	Grid2:LogEventStart("table 10 items")
	local x = 0
	for i = 1, 10000, 1 do
		for j, spellId in ipairs(table0) do
			if (t1[spellId]) then
				x = x + 1
			end
		end
		for j, spellId in ipairs(table1) do
			if (t1[spellId]) then
				x = x + 1
			end
		end
	end
	Grid2:LogEventEnd("table 10 items", x, "accesses")
	x = 0
	Grid2:LogEventStart("table 35 items")
	for i = 1, 10000, 1 do
		for j, spellId in ipairs(table0) do
			if (t2[spellId]) then
				x = x + 1
			end
		end
		for j, spellId in ipairs(table1) do
			if (t2[spellId]) then
				x = x + 1
			end
		end
	end
	Grid2:LogEventEnd("table 35 items", x, "accesses")
end

-- Hook SetupAuraStatus to all StatusAuraGroup in the setup
local prev_SetupAuraStatus = Grid2.SetupAuraStatus
function Grid2:SetupAuraStatus(setup, ...)
	prev_SetupAuraStatus(self, setup, ...)

	local status
	for statusKey, infoAuraGroup in pairs(setup.auraGroups) do
		status = self:SetupStatusAuraGroupBuff(infoAuraGroup)
		self:SetupStatusAuraGroup(status, statusKey, infoAuraGroup)
	end
	for statusKey, infoAuraGroup in pairs(setup.auraGroupDebuffs) do
		status = self:SetupStatusAuraGroupDebuff(infoAuraGroup)
		self:SetupStatusAuraGroup(status, statusKey, infoAuraGroup)
	end

--test()
end

--}}}


-- Hook the loading of options so our associated lod options get loaded at the right time.
local prev_LoadOptions = Grid2.LoadOptions
function Grid2:LoadOptions(...)
	prev_LoadOptions(self, ...)
	if Grid2StatusAuraGroupOptions then return end
	if not IsAddOnLoaded("Grid2StatusAuraGroupOptions") then
		LoadAddOn("Grid2StatusAuraGroupOptions")
		if Grid2StatusAuraGroupOptions then
			Grid2StatusAuraGroupOptions:Initialize()
		end
	end
end
